# python script to create the input file used to compute finite difference

import sys
import os
from systeme import system
from datetime import datetime
try:
  import numpy as N
except ImportError:
  import warnings
  warnings.warn("The numpy module is missing!")
  raise
from numpy import zeros
try:
  import netCDF4 as nc
except ImportError:
  import warnings
  warnings.warn("The netCDF4 module is missing!")
  raise

start = datetime.now()
print 'Start on %s/%s/%s at %sh%s ' %(start.day,start.month,start.year,start.hour,start.minute)


Ha2eV = 27.21138386
kb_HaK = 3.1668154267112283e-06

# Get the path of the DDB files from user
DDB_files = []
user_input = raw_input('Enter the name of the DDB file\n')
if len(user_input.split()) != 1:
  raise Exception("You should provide only 1 file")
else: # Append and TRIM the input string with STRIP
  DDB_files.append(user_input.strip(' \t\n\r'))

DDB = system()
DDB.__init__(directory='.',filename=DDB_files[0])


# Calcul of gprimd from rprimd
rprimd = DDB.rprim*DDB.acell
gprimd = N.linalg.inv(N.matrix(rprimd))

# Transform from 2nd-order matrix (non-cartesian coordinates, 
# masses not included, asr not included ) from DDB to
# dynamical matrix, in cartesian coordinates, asr not imposed.
IFC_cart = zeros((3,DDB.natom,3,DDB.natom),dtype=complex)
for ii in N.arange(DDB.natom):
  for jj in N.arange(DDB.natom):
    for dir1 in N.arange(3):
      for dir2 in N.arange(3):
        for dir3 in N.arange(3):
          for dir4 in N.arange(3):
            IFC_cart[dir1,ii,dir2,jj] += gprimd[dir1,dir3]*DDB.IFC[dir3,ii,dir4,jj] \
          *gprimd[dir2,dir4]

# Reduce the 4 dimensional IFC_cart matrice to 2 dimensional Dynamical matrice.
ipert1 = 0
Dyn_mat = zeros((3*DDB.natom,3*DDB.natom),dtype=complex)
while ipert1 < 3*DDB.natom:
  for ii in N.arange(DDB.natom):
    for dir1 in N.arange(3):
      ipert2 = 0
      while ipert2 < 3*DDB.natom:
        for jj in N.arange(DDB.natom):
          for dir2 in N.arange(3):
            Dyn_mat[ipert1,ipert2] = IFC_cart[dir1,ii,dir2,jj]
            ipert2 += 1
      ipert1 += 1
# Hermitianize the dynamical matrix
dynmat = N.matrix(Dyn_mat)
dynmat = 0.5*(dynmat + dynmat.transpose().conjugate())

# Solve the eigenvalue problem with linear algebra (Diagonalize the matrix)
[eigval,eigvect]=N.linalg.eigh(Dyn_mat)

# Orthonormality relation 
#eigvect = (eigvect)*N.sqrt(5.4857990965007152E-4/float(DDB.amu[0]))
kk = 0
for jj in eigval:
  if jj < 0.0:
    #print "WARNING: An eigenvalue is negative with value: ",jj
    eigval[kk] = 0.0
    kk += 1
  else:
    kk += 1
omega = N.sqrt((eigval*5.4857990965007152E-4)/float(DDB.amu[0]))

#DBSP 
#eigvect = N.array([[0,0,0,0,0,0],[0,0,0,0,0,0],[0,0,0,0,0,0],[1.0/N.sqrt(6),1.0/N.sqrt(6),1.0/N.sqrt(6),-1.0/N.sqrt(6),-1.0/N.sqrt(6),-1.0/N.sqrt(6)]])
#END

eigvect = eigvect.real
eigvect = eigvect.transpose()
print "Eigenvect: ",eigvect.real  # Here real because q=0
print "Phonon freq DFPT: ",omega

acell = 6.67527613817
h = 0.08
#Romberg order
order = 12


ll=1
print "xcart1    3*0.0      3*"+str(acell*0.25)+"   #f(x1,x2)"
for ii in N.arange(1,order+1):
  print "#h: ",h
#DBSP
#  print "xcart%d %0.15f %0.15f %0.15f %0.15f %0.15f %0.15f #f(x1+h,x2+h)" %(ll+1,h*eigvect[4,0],h*eigvect[4,1],h*eigvect[4,2],acell*0.25+h*eigvect[4,3],acell*0.25+h*eigvect[4,4],acell*0.25+h*eigvect[4,5])  # 1st optical mode
#  print "xcart%d %0.15f %0.15f %0.15f %0.15f %0.15f %0.15f #f(x1-h,x2-h)" %(ll+2,-h*eigvect[4,0],-h*eigvect[4,1],-h*eigvect[4,2],acell*0.25-h*eigvect[4,3],acell*0.25-h*eigvect[4,4],acell*0.25-h*eigvect[4,5])  # 1st optical mode
#  print "xcart%d %0.15f %0.15f %0.15f %0.15f %0.15f %0.15f #f(x1+h,x2+h)" %(ll+1,h*eigvect[5,0],h*eigvect[5,1],h*eigvect[5,2],acell*0.25+h*eigvect[5,3],acell*0.25+h*eigvect[5,4],acell*0.25+h*eigvect[5,5])  # 1st optical mode
#  print "xcart%d %0.15f %0.15f %0.15f %0.15f %0.15f %0.15f #f(x1-h,x2-h)" %(ll+2,-h*eigvect[5,0],-h*eigvect[5,1],-h*eigvect[5,2],acell*0.25-h*eigvect[5,3],acell*0.25-h*eigvect[5,4],acell*0.25-h*eigvect[5,5])  # 1st optical mode
  print "# Mode LO1"
  print "xcart%d %0.15f %0.15f %0.15f %0.15f %0.15f %0.15f #f(x1+h,x2+h)" %(ll+1,h*eigvect[3,0],h*eigvect[3,1],h*eigvect[3,2],acell*0.25+h*eigvect[3,3],acell*0.25+h*eigvect[3,4],acell*0.25+h*eigvect[3,5])  # 1st optical mode
  print "xcart%d %0.15f %0.15f %0.15f %0.15f %0.15f %0.15f #f(x1-h,x2-h)" %(ll+2,-h*eigvect[3,0],-h*eigvect[3,1],-h*eigvect[3,2],acell*0.25-h*eigvect[3,3],acell*0.25-h*eigvect[3,4],acell*0.25-h*eigvect[3,5])  # 1st optical mode
  ll += 2  
  print "# Mode LO2"
  print "xcart%d %0.15f %0.15f %0.15f %0.15f %0.15f %0.15f #f(x1+h,x2+h)" %(ll+1,h*eigvect[4,0],h*eigvect[4,1],h*eigvect[4,2],acell*0.25+h*eigvect[4,3],acell*0.25+h*eigvect[4,4],acell*0.25+h*eigvect[4,5])  # 1st optical mode
  print "xcart%d %0.15f %0.15f %0.15f %0.15f %0.15f %0.15f #f(x1-h,x2-h)" %(ll+2,-h*eigvect[4,0],-h*eigvect[4,1],-h*eigvect[4,2],acell*0.25-h*eigvect[4,3],acell*0.25-h*eigvect[4,4],acell*0.25-h*eigvect[4,5])  # 1st optical mode
  ll += 2  
  print "# Mode LO3"
  print "xcart%d %0.15f %0.15f %0.15f %0.15f %0.15f %0.15f #f(x1+h,x2+h)" %(ll+1,h*eigvect[5,0],h*eigvect[5,1],h*eigvect[5,2],acell*0.25+h*eigvect[5,3],acell*0.25+h*eigvect[5,4],acell*0.25+h*eigvect[5,5])  # 1st optical mode
  print "xcart%d %0.15f %0.15f %0.15f %0.15f %0.15f %0.15f #f(x1-h,x2-h)" %(ll+2,-h*eigvect[5,0],-h*eigvect[5,1],-h*eigvect[5,2],acell*0.25-h*eigvect[5,3],acell*0.25-h*eigvect[5,4],acell*0.25-h*eigvect[5,5])  # 1st optical mode
#END
  print "# ---------------------"  

  ll += 2  
  h = h/2



