# Python script to compute finite differences
import numpy as np
import netCDF4 as nc
import sys
import os

# Read _EIG.nc file
class open_file:
  filename = None
  filefullpath = None
  def __init__(self,directory=None,filename=None):
    if filename == None:return
    if directory == None:directory='.'
    self.filename = filename
    self.filefullpath = '%s/%s' %(directory,filename)
    if self.filefullpath[-7:] == '_EIG.nc':
      self.EIG_file_open(self.filefullpath)

  def EIG_file_open(self,filefullpath):
    if not (os.path.isfile(filefullpath)):
      raise Exception('The file "%s" does not exists!' %filefullpath)
    root = nc.Dataset(filefullpath,'r')
    self.EIG = root.variables['Eigenvalues'][:,:]
    self.Kptns = root.variables['Kptns'][:,:]
    NBandK = root.variables['NBandK'][:]
    self.nband =  np.int(NBandK[0,0])
    root.close()

# Carbon mass in a.u.
Mc = 21894.16693

# Frequency (computed in DFPT using ABINIT)
omega = np.array([1.725945E-05,1.730365E-05,1.787263E-05,0.006064719,0.006064719,0.006064719 ])
omega[3] = 6.064741669E-03


#-------------------------------------------------------------------------
print "This script compute the ZPR through finite difference using a Richardson extrapolation"
user_input = raw_input('Richardson extrapolation order: \n')
try:
  order = int(user_input)
except ValueError:
  raise Exception('The value you enter is not an integer!')

user_input = raw_input('Richardson steps e.g.: 0.02 0.01 0.005 0.0025 0.00125\n')
tmp = user_input.split() 
h = np.zeros(order)
for ii in np.arange(0,order):
  h[ii] = np.float(tmp[ii])

D = np.zeros([len(h),len(h),4])

print "Now provide _EIG.nc files starting with the equilibrium case"
eig_files = []
for ii in np.arange(order*6+1):
  user_input = raw_input('Enter the name of the %s _EIG.nc file\n' %ii)
  if len(user_input.split()) != 1:
    raise Exception("You should provide only 1 file")
  else:
    eig_files.append(user_input.strip(' \t\n\r'))

preterm = (27.211383*1000)/(4*Mc*omega[3])
eigenEq = open_file()
eigenEq.__init__(directory='.',filename=eig_files[0])
ii = 1
while ii < order*6+1:
  eigen1 = open_file()
  eigen1.__init__(directory='.',filename=eig_files[ii])
  eigen2 = open_file()
  eigen2.__init__(directory='.',filename=eig_files[ii+1])
  eigen3 = open_file()
  eigen3.__init__(directory='.',filename=eig_files[ii+2])
  eigen4 = open_file()
  eigen4.__init__(directory='.',filename=eig_files[ii+3])
  eigen5 = open_file()
  eigen5.__init__(directory='.',filename=eig_files[ii+4])
  eigen6 = open_file()
  eigen6.__init__(directory='.',filename=eig_files[ii+5])
#  total = 3*(preterm*(np.sum(eigen1.EIG[0,0,1:4])/3+np.sum(eigen2.EIG[0,0,1:4])/3-(2*np.sum(eigenEq.EIG[0,0,1:4]))/3)/(h[(ii-1)/2]**2)) #Times 3 because 3 optical modes

# Range
  range = [0] #  for HOMO level 
  kpt  = 0
# HOMO
#  D[0,(ii-1)/2,0] = 3*(preterm*(np.sum(eigen1.EIG[0,0,1:4])/3+np.sum(eigen2.EIG[0,0,1:4])/3-(2*np.sum(eigenEq.EIG[0,0,1:4]))/3)/(h[(ii-1)/2]**2))
# LUMO
#  D[0,(ii-1)/2,0] = 3*(preterm*(np.sum(eigen1.EIG[0,0,4:7])/3+np.sum(eigen2.EIG[0,0,4:7])/3-(2*np.sum(eigenEq.EIG[0,0,4:7]))/3)/(h[(ii-1)/2]**2))
#  print eigen1.Kptns[3,:]
  D[0,(ii-1)/6,0] = (preterm*(np.sum(eigen1.EIG[0,kpt,range])+np.sum(eigen2.EIG[0,kpt,range])-(2*np.sum(eigenEq.EIG[0,kpt,range])))/(h[(ii-1)/6]**2))
  D[0,(ii-1)/6,1] = (preterm*(np.sum(eigen3.EIG[0,kpt,range])+np.sum(eigen4.EIG[0,kpt,range])-(2*np.sum(eigenEq.EIG[0,kpt,range])))/(h[(ii-1)/6]**2))
  D[0,(ii-1)/6,2] = (preterm*(np.sum(eigen5.EIG[0,kpt,range])+np.sum(eigen6.EIG[0,kpt,range])-(2*np.sum(eigenEq.EIG[0,kpt,range])))/(h[(ii-1)/6]**2))

#  print "h: ",h[(ii-1)/2] 
#  print "R0+h+h: ",str(np.sum(eigen1.EIG[0,0,1:4])/3)
#  print "R0-h-h: ",str(np.sum(eigen2.EIG[0,0,1:4])/3)
#  print "R0: ",str(np.sum(eigenEq.EIG[0,0,1:4])/3)
  ii += 6



# ------------------------------------------------------------------------
# We will now use the Richardson Extrapolation to speed up the convergence
# ------------------------------------------------------------------------

l = 0
while l < len(h):
  k = 0
  while k < len(h):
    if (l-1) >= 0 and (k-1) >= 0:
#     LO mode 1  
      D[l,k,0] = (D[l-1,k,0]-(D[l-1,k-1,0]*(1.0/(2**(2*l)))))/\
                 (1-(1.0/(2**(2*l))))
#     LO mode 2
      D[l,k,1] = (D[l-1,k,1]-(D[l-1,k-1,1]*(1.0/(2**(2*l)))))/\
                 (1-(1.0/(2**(2*l))))
#     LO mode 3  
      D[l,k,2] = (D[l-1,k,2]-(D[l-1,k-1,2]*(1.0/(2**(2*l)))))/\
                 (1-(1.0/(2**(2*l))))
      if D[l-1,k-1,0] == 0.0:
        D[l,k,0] = 0.0
      if D[l-1,k-1,1] == 0.0:
        D[l,k,1] = 0.0
      if D[l-1,k-1,2] == 0.0:
        D[l,k,2] = 0.0
    k += 1
  l += 1

print D

import matplotlib.pyplot as plt

#plt.plot(h[:],D[0,:,0])
#plt.show()

