# python script to create the input file used to compute finite difference

import sys
import os
from systeme import system
from datetime import datetime
try:
  import numpy as np
except ImportError:
  import warnings
  warnings.warn("The numpy module is missing!")
  raise
from numpy import zeros
try:
  import netCDF4 as nc
except ImportError:
  import warnings
  warnings.warn("The netCDF4 module is missing!")
  raise

start = datetime.now()
print 'Start on %s/%s/%s at %sh%s ' %(start.day,start.month,start.year,start.hour,start.minute)


def red2cart(red,rprimd):
  return np.matrix(red)*rprimd


Ha2eV = 27.21138386
kb_HaK = 3.1668154267112283e-06

# Get the path of the DDB files from user
DDB_files = []
user_input = raw_input('Enter the name of the DDB file\n')
if len(user_input.split()) != 1:
  raise Exception("You should provide only 1 file")
else: # Append and TRIM the input string with STRIP
  DDB_files.append(user_input.strip(' \t\n\r'))

DDB = system()
DDB.__init__(directory='.',filename=DDB_files[0])


# Calcul of gprimd from rprimd
rprimd = DDB.rprim*DDB.acell
gprimd = np.linalg.inv(np.matrix(rprimd))

# Transform from 2nd-order matrix (non-cartesian coordinates, 
# masses not included, asr not included ) from DDB to
# dynamical matrix, in cartesian coordinates, asr not imposed.
IFC_cart = zeros((3,DDB.natom,3,DDB.natom),dtype=complex)
for ii in np.arange(DDB.natom):
  for jj in np.arange(DDB.natom):
    for dir1 in np.arange(3):
      for dir2 in np.arange(3):
        for dir3 in np.arange(3):
          for dir4 in np.arange(3):
            IFC_cart[dir1,ii,dir2,jj] += gprimd[dir1,dir3]*DDB.IFC[dir3,ii,dir4,jj] \
          *gprimd[dir2,dir4]

# Reduce the 4 dimensional IFC_cart matrice to 2 dimensional Dynamical matrice.
ipert1 = 0
Dyn_mat = zeros((3*DDB.natom,3*DDB.natom),dtype=complex)
while ipert1 < 3*DDB.natom:
  for ii in np.arange(DDB.natom):
    for dir1 in np.arange(3):
      ipert2 = 0
      while ipert2 < 3*DDB.natom:
        for jj in np.arange(DDB.natom):
          for dir2 in np.arange(3):
            Dyn_mat[ipert1,ipert2] = IFC_cart[dir1,ii,dir2,jj]
            ipert2 += 1
      ipert1 += 1
# Hermitianize the dynamical matrix
dynmat = np.matrix(Dyn_mat)
dynmat = 0.5*(dynmat + dynmat.transpose().conjugate())

# Solve the eigenvalue problem with linear algebra (Diagonalize the matrix)
[eigval,eigvect]=np.linalg.eigh(Dyn_mat)

# Orthonormality relation 
#eigvect = (eigvect)*N.sqrt(5.4857990965007152E-4/float(DDB.amu[0]))
kk = 0
for jj in eigval:
  if jj < 0.0:
    #print "WARNING: An eigenvalue is negative with value: ",jj
    eigval[kk] = 0.0
    kk += 1
  else:
    kk += 1
omega = np.sqrt((eigval*5.4857990965007152E-4)/float(DDB.amu[0]))


# eigvect = eigvect.real
eigvect = eigvect.transpose()
print "Eigenvect: ",eigvect  # Here real because q=0
print "Phonon freq DFPT: ",omega

eigvect = eigvect.real
ta1 = np.array([1,0.5,0.5])/(np.sqrt(2)*np.linalg.norm((np.array([1,0.5,0.5]))))
ta2 = np.cross(ta1,eigvect[2,0:3],axis=0)*np.sqrt(2)
#ta2 = ta2/(np.linalg.norm(ta2)*np.sqrt(2))

eigvect[0,0:3] = ta1
eigvect[0,3:6] = ta1
eigvect[1,0:3] = ta2
eigvect[1,3:6] = ta2
eigvect[3,0:3] = ta1
eigvect[3,3:6] = -ta1
eigvect[4,0:3] = ta2
eigvect[4,3:6] = -ta2


print "Eigenvect: ",eigvect
print "norm",np.linalg.norm(eigvect[0,:])
print "norm",np.linalg.norm(eigvect[1,:])
print "norm",np.linalg.norm(eigvect[2,:])

acell = np.array([1.0,1.0,1.0])*6.67527613817*2 # Supercell in the three dimension to have a good kpoint mapping. 
                                                # but a supercell 2x1x1 should be enough. 

rprim = np.array([[0.0,0.5,0.5],[0.5,0.0,0.5],[0.5,0.5,0.0]])
rprimd = np.matrix(rprim*acell)

h = 0.02
#Romberg order
order = 4

at = np.zeros((16,3))
at[0,:] = red2cart([0.0,0.0,0.0],rprimd)
at[1,:] = red2cart([1.0/8,1.0/8,1.0/8],rprimd)
at[2,:] = red2cart([1.0/2,0.0,0.0],rprimd)
at[3,:] = red2cart([5.0/8,1.0/8,1.0/8],rprimd)

at[4,:] = red2cart([0.0,1.0/2,0.0],rprimd)
at[5,:] = red2cart([1.0/8,5.0/8,1.0/8],rprimd)
at[6,:] = red2cart([1.0/2,1.0/2,0.0],rprimd)
at[7,:] = red2cart([5.0/8,5.0/8,1.0/8],rprimd)

at[8,:] = red2cart([0.0,0.0,1.0/2],rprimd)
at[9,:] = red2cart([1.0/8,1.0/8,5.0/8],rprimd)
at[10,:] = red2cart([1.0/2,0.0,1.0/2],rprimd)
at[11,:] = red2cart([5.0/8,1.0/8,5.0/8],rprimd)

at[12,:] = red2cart([0.0,1.0/2,1.0/2],rprimd)
at[13,:] = red2cart([1.0/8,5.0/8,5.0/8],rprimd)
at[14,:] = red2cart([1.0/2,1.0/2,1.0/2],rprimd)
at[15,:] = red2cart([5.0/8,5.0/8,5.0/8],rprimd)

name = 'finite.in'
input = '''
chkprim     0
getwfk -1
kptopt 3
prtvhxc  1

acell 13.35055227634 13.35055227634 13.35055227634

tolvrs   1.0d-18      # SCF stopping criterio
iscf       7
ngkpt     3 3 3

nshiftk     1
shiftk    0.0 0.0 0.0

# sell dependant parameters
 nbdbuf 4
 rprim 0 .5 .5 .5 0 .5 .5 .5 0
 nsym 1
 natom 16
 typat 16*1
 nband 60
 ntypat 1
 znucl 6
 diemac 6
 ecut 30
 enunit 2
 nstep 200
 istwfk *1
 paral_kgb 0'''

os.system('rm '+str(name))
os.system('touch '+str(name))
with open(name,'a') as N:
  N.write('# C in diamond\n\n ndtset \n\n')
  N.write('# For finite difference total \n\n')
  N.write( "xcart1   %0.15f %0.15f %0.15f \n" %(at[0,0],at[0,1],at[0,2]))  
  for kk in np.arange(1,16):
    N.write("         %0.15f %0.15f %0.15f \n" %(at[kk,0],at[kk,1],at[kk,2])) 

  ll=2
  for ii in np.arange(1,order+1):
    N.write( "#h: "+str(h)+"\n")
    for jj in np.arange(0,6): # Modes
      N.write( "# Mode: "+str(jj)+"\n")
      N.write( "xcart"+str(ll)+str("\n"))
      for kk in np.arange(0,4):
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk,0]+h*eigvect[jj,0],at[4*kk,1]+h*eigvect[jj,1],at[4*kk,2]+h*eigvect[jj,2]))
        N.write( "         %0.15f %0.15f %0.15f \n" %(at[4*kk+1,0]+h*eigvect[jj,3],at[4*kk+1,1]+h*eigvect[jj,4],at[4*kk+1,2]+h*eigvect[jj,5]))
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk+2,0]-h*eigvect[jj,0],at[4*kk+2,1]-h*eigvect[jj,1],at[4*kk+2,2]-h*eigvect[jj,2]))
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk+3,0]-h*eigvect[jj,3],at[4*kk+3,1]-h*eigvect[jj,4],at[4*kk+3,2]-h*eigvect[jj,5]))
      ll += 1 
    N.write( "# --------------------- \n")  

    h = h/2

with open(name,'a') as N:
  N.write(input)
 
h = 0.02
#Romberg order
order = 4

name = 'supercell.in'
os.system('rm '+str(name))
os.system('touch '+str(name))
with open(name,'a') as N:
  N.write('# C in diamond\n\n ndtset \n\n')
  N.write('# For finite difference total \n')
  N.write( "# VHXC(h,h) \n\n")
  N.write( "xcart1   %0.15f %0.15f %0.15f \n" %(at[0,0],at[0,1],at[0,2]))   
  for kk in np.arange(1,16):
    N.write( "         %0.15f %0.15f %0.15f \n" %(at[kk,0],at[kk,1],at[kk,2]))   

  ll=2
  for ii in np.arange(1,order+1):
    N.write( "#h: "+str(h)+"\n")
    for jj in np.arange(0,6): # Modes
      N.write( "# Mode: "+str(jj)+"\n")
      N.write( "# (+h,+h)\n")
      N.write( "xcart%d \n" %(ll))
      for kk in np.arange(0,4):
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk,0]+h*eigvect[jj,0],at[4*kk,1]+h*eigvect[jj,1],at[4*kk,2]+h*eigvect[jj,2]))
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk+1,0]+h*eigvect[jj,3],at[4*kk+1,1]+h*eigvect[jj,4],at[4*kk+1,2]+h*eigvect[jj,5]))
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk+2,0]-h*eigvect[jj,0],at[4*kk+2,1]-h*eigvect[jj,1],at[4*kk+2,2]-h*eigvect[jj,2]))
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk+3,0]-h*eigvect[jj,3],at[4*kk+3,1]-h*eigvect[jj,4],at[4*kk+3,2]-h*eigvect[jj,5]))
      ll += 1 
      N.write( "# (+h,-h)\n")
      N.write( "xcart%d \n" %(ll))
      for kk in np.arange(0,4):
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk,0]+h*eigvect[jj,0],at[4*kk,1]+h*eigvect[jj,1],at[4*kk,2]+h*eigvect[jj,2]))
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk+1,0]-h*eigvect[jj,3],at[4*kk+1,1]-h*eigvect[jj,4],at[4*kk+1,2]-h*eigvect[jj,5]))
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk+2,0]-h*eigvect[jj,0],at[4*kk+2,1]-h*eigvect[jj,1],at[4*kk+2,2]-h*eigvect[jj,2]))
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk+3,0]+h*eigvect[jj,3],at[4*kk+3,1]+h*eigvect[jj,4],at[4*kk+3,2]+h*eigvect[jj,5]))
      ll += 1
      N.write( "# (-h,+h)\n")
      N.write( "xcart%d \n" %(ll))
      for kk in np.arange(0,4):
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk,0]-h*eigvect[jj,0],at[4*kk,1]-h*eigvect[jj,1],at[4*kk,2]-h*eigvect[jj,2]))
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk+1,0]+h*eigvect[jj,3],at[4*kk+1,1]+h*eigvect[jj,4],at[4*kk+1,2]+h*eigvect[jj,5]))
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk+2,0]+h*eigvect[jj,0],at[4*kk+2,1]+h*eigvect[jj,1],at[4*kk+2,2]+h*eigvect[jj,2]))
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk+3,0]-h*eigvect[jj,3],at[4*kk+3,1]-h*eigvect[jj,4],at[4*kk+3,2]-h*eigvect[jj,5]))
      ll += 1
      N.write( "# (-h,-h)\n")
      N.write( "xcart%d \n" %(ll))
      for kk in np.arange(0,4):
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk,0]-h*eigvect[jj,0],at[4*kk,1]-h*eigvect[jj,1],at[4*kk,2]-h*eigvect[jj,2]))
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk+1,0]-h*eigvect[jj,3],at[4*kk+1,1]-h*eigvect[jj,4],at[4*kk+1,2]-h*eigvect[jj,5]))
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk+2,0]+h*eigvect[jj,0],at[4*kk+2,1]+h*eigvect[jj,1],at[4*kk+2,2]+h*eigvect[jj,2]))
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk+3,0]+h*eigvect[jj,3],at[4*kk+3,1]+h*eigvect[jj,4],at[4*kk+3,2]+h*eigvect[jj,5]))
      ll += 1
      N.write( "# (+h,0)\n")
      N.write( "xcart%d \n" %(ll))
      for kk in np.arange(0,4):
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk,0]+h*eigvect[jj,0],at[4*kk,1]+h*eigvect[jj,1],at[4*kk,2]+h*eigvect[jj,2]))
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk+1,0],at[4*kk+1,1],at[4*kk+1,2]))
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk+2,0]-h*eigvect[jj,0],at[4*kk+2,1]-h*eigvect[jj,1],at[4*kk+2,2]-h*eigvect[jj,2]))
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk+3,0],at[4*kk+3,1],at[4*kk+3,2]))
      ll += 1
      N.write( "# (-h,0)\n")
      N.write("xcart"+str(ll)+"\n")
      for kk in np.arange(0,4):
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk,0]-h*eigvect[jj,0],at[4*kk,1]-h*eigvect[jj,1],at[4*kk,2]-h*eigvect[jj,2]))
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk+1,0],at[4*kk+1,1],at[4*kk+1,2]))
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk+2,0]+h*eigvect[jj,0],at[4*kk+2,1]+h*eigvect[jj,1],at[4*kk+2,2]+h*eigvect[jj,2]))
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk+3,0],at[4*kk+3,1],at[4*kk+3,2]))
      ll += 1
      N.write( "# (0,+h)\n")
      N.write( "xcart%d \n" %(ll))
      for kk in np.arange(0,4):
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk,0],at[4*kk,1],at[4*kk,2]))
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk+1,0]+h*eigvect[jj,3],at[4*kk+1,1]+h*eigvect[jj,4],at[4*kk+1,2]+h*eigvect[jj,5]))
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk+2,0],at[4*kk+2,1],at[4*kk+2,2]))
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk+3,0]-h*eigvect[jj,3],at[4*kk+3,1]-h*eigvect[jj,4],at[4*kk+3,2]-h*eigvect[jj,5]))
      ll += 1
      N.write( "# (0,-h)\n")
      N.write( "xcart%d \n" %(ll))
      for kk in np.arange(0,4):
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk,0],at[4*kk,1],at[4*kk,2]))
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk+1,0]-h*eigvect[jj,3],at[4*kk+1,1]-h*eigvect[jj,4],at[4*kk+1,2]-h*eigvect[jj,5]))
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk+2,0],at[4*kk+2,1],at[4*kk+2,2]))
        N.write( "        %0.15f %0.15f %0.15f \n" %(at[4*kk+3,0]+h*eigvect[jj,3],at[4*kk+3,1]+h*eigvect[jj,4],at[4*kk+3,2]+h*eigvect[jj,5]))
      ll += 1
    N.write( "# --------------------- \n" )
    h = h/2
 

with open(name,'a') as N:
  N.write(input) 
