# Make all the calculation using Paul python scripts
import netCDF4 as nc
import numpy as np
import math
import os

# User defined variables
band = 1 #  for 1  level 
range = np.array([0])
kpt  = 1 # G point
ii= 2
order = 4
h = np.array([0.02, 0.01, 0.005, 0.0025])
D = np.zeros([len(h),len(h),7])
DD = np.zeros([len(h),len(h),7])
Mc = 21894.16693
# Frequency (computed in DFPT using ABINIT)
omega = np.array([2.502563E-03,  2.502728E-03,  4.941578E-03,  5.647860E-03,  5.647892E-03, 5.775970E-03])

def use_cut3d(datafile,out,gamma,band):
  if datafile[len(datafile)-4:len(datafile)] == "VHXC":
    os.system("rm cut3d.files")
    with open("cut3d.files","a") as files:
      files.write(str(datafile)+"\n")
      files.write("1 \n")
      files.write("5 \n")  #3D formatted data 
                           #(output the bare 3D data - one column) 
      files.write(str(out)+"\n")
      files.write("0 \n")
    os.system("cut3d < cut3d.files ")
  if datafile[len(datafile)-3:len(datafile)] == "WFK":
    os.system("rm wfk.files")
    with open("wfk.files","a") as files:
      files.write(str(datafile)+"\n")
      files.write("1 \n")
      files.write("0 \n")
      files.write(str(gamma)+" \n") # Gamma point
      files.write(str(band)+" \n") # Band nb 6
      files.write("0 \n")
      files.write("0 \n")
      files.write("2 \n") # real 3D data one column
      files.write(str(out)+" \n")
      files.write("0 \n")
    os.system("cut3d < wfk.files ")

def integration(VHXC,wfk):
  psi_file=open(wfk,'r')
  v_file=open(VHXC,'r')
  S=0.0; N=0.0
  while True:
    psi_l=psi_file.readline()
    v_l=v_file.readline()
    if not psi_l or not v_l: break
    psi=float(psi_l)
    v=float(v_l)
    S+=psi*v*psi
    N+=psi*psi
  psi_file.close()
  v_file.close()
  S=S/N
  return S

# Read _EIG.nc file
class open_file:
  filename = None
  filefullpath = None
  def __init__(self,directory=None,filename=None):
    if filename == None:return
    if directory == None:directory='.'
    self.filename = filename
    self.filefullpath = '%s/%s' %(directory,filename)
    if self.filefullpath[-7:] == '_EIG.nc':
      self.EIG_file_open(self.filefullpath)

  def EIG_file_open(self,filefullpath):
    if not (os.path.isfile(filefullpath)):
      raise Exception('The file "%s" does not exists!' %filefullpath)
    root = nc.Dataset(filefullpath,'r')
    self.EIG = root.variables['Eigenvalues'][:,:]
    self.Kptns = root.variables['Kptns'][:,:]
    NBandK = root.variables['NBandK'][:]
    self.nband =  np.int(NBandK[0,0])
    root.close()

# Full finite
eigenEq = open_file(directory='.',filename="finiteo_DS1_EIG.nc")

while ii < order*6+1:
  eigen1 = open_file(directory='.',filename="finiteo_DS"+str(ii)+"_EIG.nc")
  eigen2 = open_file(directory='.',filename="finiteo_DS"+str(ii+1)+"_EIG.nc")
  eigen3 = open_file(directory='.',filename="finiteo_DS"+str(ii+2)+"_EIG.nc")
  eigen4 = open_file(directory='.',filename="finiteo_DS"+str(ii+3)+"_EIG.nc")
  eigen5 = open_file(directory='.',filename="finiteo_DS"+str(ii+4)+"_EIG.nc")
  eigen6 = open_file(directory='.',filename="finiteo_DS"+str(ii+5)+"_EIG.nc")

  preterm = (1.0/(4*Mc*omega[0]))*27.211383*1000
  D[0,(ii-1)/6,0] = 2*preterm*(np.sum(eigen1.EIG[0,kpt,range])/(len(range))-np.sum(eigenEq.EIG[0,kpt,range]/(len(range))))/(h[(ii-1)/6]**2)
  preterm = (1.0/(4*Mc*omega[1]))*27.211383*1000
  D[0,(ii-1)/6,1] = 2*preterm*(np.sum(eigen2.EIG[0,kpt,range])/(len(range))-np.sum(eigenEq.EIG[0,kpt,range]/(len(range))))/(h[(ii-1)/6]**2)
  preterm = (1.0/(4*Mc*omega[2]))*27.211383*1000
  D[0,(ii-1)/6,2] = 2*preterm*(np.sum(eigen3.EIG[0,kpt,range])/(len(range))-np.sum(eigenEq.EIG[0,kpt,range]/(len(range))))/(h[(ii-1)/6]**2)
  preterm = (1.0/(4*Mc*omega[3]))*27.211383*1000
  D[0,(ii-1)/6,3] = 2*preterm*(np.sum(eigen4.EIG[0,kpt,range])/(len(range))-np.sum(eigenEq.EIG[0,kpt,range]/(len(range))))/(h[(ii-1)/6]**2)
  preterm = (1.0/(4*Mc*omega[4]))*27.211383*1000
  D[0,(ii-1)/6,4] = 2*preterm*(np.sum(eigen5.EIG[0,kpt,range])/(len(range))-np.sum(eigenEq.EIG[0,kpt,range]/(len(range))))/(h[(ii-1)/6]**2)
  preterm = (1.0/(4*Mc*omega[5]))*27.211383*1000
  D[0,(ii-1)/6,5] = 2*preterm*(np.sum(eigen6.EIG[0,kpt,range])/(len(range))-np.sum(eigenEq.EIG[0,kpt,range]/(len(range))))/(h[(ii-1)/6]**2)
  D[0,(ii-1)/6,6] = np.sum(D[0,(ii-1)/6,0:6]) # Sum over modes
 
  print "omega", omega[2] 
  print "D LA componant: ", D[0,(ii-1)/6,2]
  ii += 6

# ------------------------------------------------------------------------
# We will now use the Richardson Extrapolation to speed up the convergence
# ------------------------------------------------------------------------

l = 0
while l < len(h):
  k = 0
  while k < len(h):
    if (l-1) >= 0 and (k-1) >= 0:
#      mode 1  
      D[l,k,0] = (D[l-1,k,0]-(D[l-1,k-1,0]*(1.0/(2**(2*l)))))/\
                 (1-(1.0/(2**(2*l))))
#      mode 2
      D[l,k,1] = (D[l-1,k,1]-(D[l-1,k-1,1]*(1.0/(2**(2*l)))))/\
                 (1-(1.0/(2**(2*l))))
#      mode 3  
      D[l,k,2] = (D[l-1,k,2]-(D[l-1,k-1,2]*(1.0/(2**(2*l)))))/\
                 (1-(1.0/(2**(2*l))))
#      mode 4  
      D[l,k,3] = (D[l-1,k,3]-(D[l-1,k-1,3]*(1.0/(2**(2*l)))))/\
                 (1-(1.0/(2**(2*l))))
#      mode 5  
      D[l,k,4] = (D[l-1,k,4]-(D[l-1,k-1,4]*(1.0/(2**(2*l)))))/\
                 (1-(1.0/(2**(2*l))))
#      mode 6  
      D[l,k,5] = (D[l-1,k,5]-(D[l-1,k-1,5]*(1.0/(2**(2*l)))))/\
                 (1-(1.0/(2**(2*l))))
#     sum modes  
      D[l,k,6] = (D[l-1,k,6]-(D[l-1,k-1,6]*(1.0/(2**(2*l)))))/\
                 (1-(1.0/(2**(2*l))))
      if D[l-1,k-1,0] == 0.0:
        D[l,k,0] = 0.0
      if D[l-1,k-1,1] == 0.0:
        D[l,k,1] = 0.0
      if D[l-1,k-1,2] == 0.0:
        D[l,k,2] = 0.0
      if D[l-1,k-1,3] == 0.0:
        D[l,k,3] = 0.0
      if D[l-1,k-1,4] == 0.0:
        D[l,k,4] = 0.0
      if D[l-1,k-1,5] == 0.0:
        D[l,k,5] = 0.0
      if D[l-1,k-1,6] == 0.0:
        D[l,k,6] = 0.0
    k += 1
  l += 1

finite = D[:,:,6]

# NRIA
#use_cut3d("supercello_DS1_VHXC","EQ",0,0)
#use_cut3d("supercello_DS1_WFK","wfk",1,1)
#EQ  = integration("EQ","wfk_k1_b1_s1")
#
#ii = 2
#while ii < order*48+1:
#  for jj in np.arange(0,6):
#    use_cut3d("supercello_DS"+str(8*jj+ii)+"_VHXC","V1",0,0)
#    V1 = integration("V1","wfk_k1_b1_s1")      
#    use_cut3d("supercello_DS"+str(8*jj+ii+1)+"_VHXC","V2",0,0)
#    V2 = integration("V2","wfk_k1_b1_s1")     
#    use_cut3d("supercello_DS"+str(8*jj+ii+2)+"_VHXC","V3",0,0)
#    V3 = integration("V3","wfk_k1_b1_s1")     
#    use_cut3d("supercello_DS"+str(8*jj+ii+3)+"_VHXC","V4",0,0)
#    V4 = integration("V4","wfk_k1_b1_s1")     
#    preterm = (1.0/(4*Mc*omega[jj]))*27.211383*1000
#    D[0,(ii-1)/48,jj] = 2*preterm*(V1+V4-V2-V3)/(4*(h[(ii-1)/48]**2))
#    use_cut3d("supercello_DS"+str(8*jj+ii+4)+"_VHXC","VV1",0,0)
#    VV1 = integration("VV1","wfk_k1_b1_s1")
#    use_cut3d("supercello_DS"+str(8*jj+ii+5)+"_VHXC","VV2",0,0)
#    VV2 = integration("VV2","wfk_k1_b1_s1")
#    use_cut3d("supercello_DS"+str(8*jj+ii+6)+"_VHXC","VV3",0,0)
#    VV3 = integration("VV3","wfk_k1_b1_s1")
#    use_cut3d("supercello_DS"+str(8*jj+ii+7)+"_VHXC","VV4",0,0)
#    VV4 = integration("VV4","wfk_k1_b1_s1")
#    DD[0,(ii-1)/48,jj] = preterm*(VV1+VV2-2*EQ)/((h[(ii-1)/48]**2)) + preterm*(VV3+VV4-2*EQ)/((h[(ii-1)/48]**2))
#
#
#  D[0,(ii-1)/48,6] = np.sum(D[0,(ii-1)/48,0:6]) # Sum over modes
#  DD[0,(ii-1)/48,6] = np.sum(DD[0,(ii-1)/48,0:6]) # Sum over modes
#
#  ii += 48
#
## ------------------------------------------------------------------------
## We will now use the Richardson Extrapolation to speed up the convergence
## ------------------------------------------------------------------------
#
#l = 0
#while l < len(h):
#  k = 0
#  while k < len(h):
#    if (l-1) >= 0 and (k-1) >= 0:
##      mode 1  
#      D[l,k,0] = (D[l-1,k,0]-(D[l-1,k-1,0]*(1.0/(2**(2*l)))))/\
#                 (1-(1.0/(2**(2*l))))
##      mode 2
#      D[l,k,1] = (D[l-1,k,1]-(D[l-1,k-1,1]*(1.0/(2**(2*l)))))/\
#                 (1-(1.0/(2**(2*l))))
##      mode 3  
#      D[l,k,2] = (D[l-1,k,2]-(D[l-1,k-1,2]*(1.0/(2**(2*l)))))/\
#                 (1-(1.0/(2**(2*l))))
##      mode 4  
#      D[l,k,3] = (D[l-1,k,3]-(D[l-1,k-1,3]*(1.0/(2**(2*l)))))/\
#                 (1-(1.0/(2**(2*l))))
##      mode 5  
#      D[l,k,4] = (D[l-1,k,4]-(D[l-1,k-1,4]*(1.0/(2**(2*l)))))/\
#                 (1-(1.0/(2**(2*l))))
##      mode 6  
#      D[l,k,5] = (D[l-1,k,5]-(D[l-1,k-1,5]*(1.0/(2**(2*l)))))/\
#                 (1-(1.0/(2**(2*l))))
##     sum modes  
#      D[l,k,6] = (D[l-1,k,6]-(D[l-1,k-1,6]*(1.0/(2**(2*l)))))/\
#                 (1-(1.0/(2**(2*l))))
#      if D[l-1,k-1,0] == 0.0:
#        D[l,k,0] = 0.0
#      if D[l-1,k-1,1] == 0.0:
#        D[l,k,1] = 0.0
#      if D[l-1,k-1,2] == 0.0:
#        D[l,k,2] = 0.0
#      if D[l-1,k-1,3] == 0.0:
#        D[l,k,3] = 0.0
#      if D[l-1,k-1,4] == 0.0:
#        D[l,k,4] = 0.0
#      if D[l-1,k-1,5] == 0.0:
#        D[l,k,5] = 0.0
#      if D[l-1,k-1,6] == 0.0:
#        D[l,k,6] = 0.0
#    k += 1
#  l += 1
#
#NDDW1 = D[:,:,6]
#
#
## ------------------------------------------------------------------------
## We will now use the Richardson Extrapolation to speed up the convergence
## ------------------------------------------------------------------------
#
#l = 0
#while l < len(h):
#  k = 0
#  while k < len(h):
#    if (l-1) >= 0 and (k-1) >= 0:
##      mode 1  
#      DD[l,k,0] = (DD[l-1,k,0]-(DD[l-1,k-1,0]*(1.0/(2**(2*l)))))/\
#                 (1-(1.0/(2**(2*l))))
##      mode 2
#      DD[l,k,1] = (DD[l-1,k,1]-(DD[l-1,k-1,1]*(1.0/(2**(2*l)))))/\
#                 (1-(1.0/(2**(2*l))))
##      mode 3  
#      DD[l,k,2] = (DD[l-1,k,2]-(DD[l-1,k-1,2]*(1.0/(2**(2*l)))))/\
#                 (1-(1.0/(2**(2*l))))
##      mode 4  
#      DD[l,k,3] = (DD[l-1,k,3]-(DD[l-1,k-1,3]*(1.0/(2**(2*l)))))/\
#                 (1-(1.0/(2**(2*l))))
##      mode 5  
#      DD[l,k,4] = (DD[l-1,k,4]-(DD[l-1,k-1,4]*(1.0/(2**(2*l)))))/\
#                 (1-(1.0/(2**(2*l))))
##      mode 6  
#      DD[l,k,5] = (DD[l-1,k,5]-(DD[l-1,k-1,5]*(1.0/(2**(2*l)))))/\
#                 (1-(1.0/(2**(2*l))))
##     sum modes  
#      DD[l,k,6] = (DD[l-1,k,6]-(DD[l-1,k-1,6]*(1.0/(2**(2*l)))))/\
#                 (1-(1.0/(2**(2*l))))
#      if DD[l-1,k-1,0] == 0.0:
#        DD[l,k,0] = 0.0
#      if DD[l-1,k-1,1] == 0.0:
#        DD[l,k,1] = 0.0
#      if DD[l-1,k-1,2] == 0.0:
#        DD[l,k,2] = 0.0
#      if DD[l-1,k-1,3] == 0.0:
#        DD[l,k,3] = 0.0
#      if DD[l-1,k-1,4] == 0.0:
#        DD[l,k,4] = 0.0
#      if DD[l-1,k-1,5] == 0.0:
#        DD[l,k,5] = 0.0
#      if DD[l-1,k-1,6] == 0.0:
#        DD[l,k,6] = 0.0
#    k += 1
#  l += 1
#
#DDW_NRIA_1 = DD[:,:,6]
#
## Phonon
#etot = np.array([-96.606738860919,-96.606728058283, -96.606735938865,-96.606685395059,-96.606690037862,-96.606717842867,-96.606665816881])
#
#preterm = (1)/(Mc)  # in Hai
#for ii in np.arange(1,7):
#  omega = (1.0/8)*2*preterm*(etot[ii]-etot[0])/(h**2) # supercell 2x2x2 = 8 times larger energies
#  print "Freq phonon ",np.sqrt(omega)
#  
#

print "finite ",finite
#print "NDDW1", NDDW1
#print "DDW_NRIA_1 ",DDW_NRIA_1 




