# ------------------------------------------------------------------
# This script genereate cartesian coordinate for Abinit input files.
# Specifically for q = L = (1/3 0.0 0.0)
# Create inputs for finite diff. on eigenenergies and NDDW
# ------------------------------------------------------------------

import numpy as np

# ------------------------------------------------------------------
# Find the eigenvector and eigenenergies from the dynamical matrix
# computed using DFPT in ABINIT
# ------------------------------------------------------------------

# Location of the Dynamical matrix file
L_file = '/home/ucl/naps/sponce/Diamond/WorkDiam_2L3/Dyanmical_mat.dat'
# Carbon mass in a.u.
Mc=21894.16693

def read_dynamical_matrix(datafile):
  with open(datafile,'r') as f:
    raw = list()
    for line in f:
      if line[0]=='#': continue
      parts=line.split()
      if not parts: continue
      valR=float(parts[4]) #real
      valI=float(parts[5]) #imaginary
      if np.abs(valR) < 0.000000001:
        valR=0.0
      if np.abs(valI) < 0.000000001:
        valI=0.0
      raw.append(complex(valR,valI))
    dynmat = np.array(raw)
    dynmat = dynmat.reshape(6,6)
  return dynmat

dynmat=read_dynamical_matrix(L_file)
[eigval,eigvect]=np.linalg.eig(dynmat)
eigvect=np.transpose(eigvect)

print "eigenvalues at 2/3L :"
for n in eigval:
  a = np.sqrt(n/Mc)
  print a

print "eigenvectors at 2/3L :"
for n in eigvect: print n

def print_2d_deriv(h1,h2,i,xred,cell,compl,geo,j):
  # h1 and h2 = scaling of the displacement
  # i = mode LA,LO,TO,TA
  # compl = eigvect.real or eigvect.imag
  g = 0

  if geo == 'cos': g = 1
  if geo == 'sin': g = 0

  # Atoms 1 and 2
  print " xcart%.0f    %.15f  %.15f  %.15f  "%(j,(xred[0]*cell)[0,0]+\
        compl[i][0]*h1*g,(xred[0]*cell)[0,1]+compl[i][1]*h1*g,\
        (xred[0]*cell)[0,2]+compl[i][2]*h1*g)
  print "           %.15f  %.15f  %.15f  "%((xred[1]*cell)[0,0]+\
        compl[i][3]*h2*g,(xred[1]*cell)[0,1]+compl[i][4]*h2*g,\
        (xred[1]*cell)[0,2]+compl[i][5]*h2*g)

  if geo == 'cos': g = -0.5
  if geo == 'sin': g = np.sqrt(3)/2

  # Atoms 3 and 4
  print "           %.15f  %.15f  %.15f  "%((xred[2]*cell)[0,0]+\
        compl[i][0]*h1*g,(xred[2]*cell)[0,1]+compl[i][1]*h1*g,\
        (xred[2]*cell)[0,2]+compl[i][2]*h1*g)
  print "           %.15f  %.15f  %.15f  "%((xred[3]*cell)[0,0]+\
        compl[i][3]*h2*g,(xred[3]*cell)[0,1]+compl[i][4]*h2*g,\
        (xred[3]*cell)[0,2]+compl[i][5]*h2*g)

  if geo == 'cos': g = -0.5
  if geo == 'sin': g = -np.sqrt(3)/2

  # Atoms 5 and 6
  print "           %.15f  %.15f  %.15f  "%((xred[4]*cell)[0,0]+\
        compl[i][0]*h1*g,(xred[4]*cell)[0,1]+compl[i][1]*h1*g,\
        (xred[4]*cell)[0,2]+compl[i][2]*h1*g)
  print "           %.15f  %.15f  %.15f  "%((xred[5]*cell)[0,0]+\
        compl[i][3]*h2*g,(xred[5]*cell)[0,1]+compl[i][4]*h2*g,\
        (xred[5]*cell)[0,2]+compl[i][5]*h2*g)
  print "  "
  j += 1
  return j

# -------------------------------------------------------------------
# Create Abinit input file for finite difference on eigen energies
# and the first part of NDDW. On a supercell 3x1x1
# -------------------------------------------------------------------

# Value and number of the displacement (always 1/2 of the previous)
h = np.array([0.01])

# Cell parameters
acell = np.array([20.110404150000001,6.70346805,6.70346805])
rprim = np.array([[0.0,0.5,0.5],[0.5,0.0,0.5],[0.5,0.5,0.0]])
cell = np.mat([rprim[0]*acell[0],rprim[1]*acell[1],rprim[2]*acell[2]])
xred = np.mat([[0.0,0.0,0.0],[1.0/12,0.25,0.25],[1.0/3,0.0,0.0],\
[5.0/12,0.25,0.25],[2.0/3,0.0,0.0],[9.0/12,0.25,0.25]])


# In the specific case of the L point, h = -h
# Indice that record the number of xcart in the input file
j = 1

# Equilibrium position
print "  "
print " Here we compute  \sum_kk' (d^2e/dRkdRk')Re(Uk)Re(Uk') "
print "Contribution of the 2/3L point on the electronic eigenenergies at Gamma"
print "and the first part of the NDDW."
print "Using finite difference."
print " "
print " xcart%.0f    %.15f  %.15f  %.15f  "%(j,(xred[0]*cell)[0,0],\
      (xred[0]*cell)[0,1],(xred[0]*cell)[0,2])
print "           %.15f  %.15f  %.15f  "%((xred[1]*cell)[0,0],\
      (xred[1]*cell)[0,1],(xred[1]*cell)[0,2])
print "           %.15f  %.15f  %.15f  "%((xred[2]*cell)[0,0],\
      (xred[2]*cell)[0,1],(xred[2]*cell)[0,2])
print "           %.15f  %.15f  %.15f  "%((xred[3]*cell)[0,0],\
      (xred[3]*cell)[0,1],(xred[3]*cell)[0,2])
print "           %.15f  %.15f  %.15f  "%((xred[4]*cell)[0,0],\
      (xred[4]*cell)[0,1],(xred[4]*cell)[0,2])
print "           %.15f  %.15f  %.15f  "%((xred[5]*cell)[0,0],\
      (xred[5]*cell)[0,1],(xred[5]*cell)[0,2])
print "  "
j += 1

# ------------------------------------------------------------------------
# We need to compute \sum_kk' (d^2e/dRkdRk')[Re(Uk)Re(Uk')+Im(Uk)Im(Uk')]
# Uk = \xi_k e^iqRk = (Re(\xi_k)+Im(\xi_k))(cosqR_k+isinqR_k)
# Re(Uk) = Re(\xi_k)cosqR_k - Im(\xi_k)sinqR_k
# Im(Uk) = Im(\xi_k)cosqR_k + Re(\xi_k)sinqR_k
# We need to make a frozen phonon and to compute the quadratic term
# -------------------------------------------------------------------------

# -------------------------------------------------------
# Here we compute  \sum_kk' (d^2e/dRkdRk')Re(Uk)Re(Uk')
# --------------------------------------------------------
mode = np.array([0,1,2,4]) #LA, LO, TA, TO

for disp in h:
# -----------------------------------------------------------------------
# We have the following phase factor: e^(q.R)=cos(2\pi/3)+isin(2\pi/3)
# So here for the real part we have:
# Atoms 1 and 2 = eq + 1*disp
# Atoms 3 and 4 = eq - 0.5*disp
# Atoms 5 and 6 = eq - 0.5*disp
# For the imaginary part we have:
# Atoms 1 and 2 = eq + 0
# Atoms 3 and 4 = eq + sqrt(3)/2*disp
# Atoms 5 and 6 = eq - sqrt(3)/2*disp
# -----------------------------------------------------------------------

 for i in mode:
  # \xiR\xiR cos cos
  # e(R0+h+h)
  j = print_2d_deriv(disp,disp,i,xred,cell,eigvect.real,'cos',j)
  # e(R0-h+h)
  j = print_2d_deriv(-disp,disp,i,xred,cell,eigvect.real,'cos',j)
  # e(R0+h-h)
  j = print_2d_deriv(disp,-disp,i,xred,cell,eigvect.real,'cos',j)
  # e(R0-h-h)
  j = print_2d_deriv(-disp,-disp,i,xred,cell,eigvect.real,'cos',j)

  # \xiR\xiR sin sin
  # e(R0+h+h)
  j = print_2d_deriv(disp,disp,i,xred,cell,eigvect.real,'sin',j)
  # e(R0-h+h)
  j = print_2d_deriv(-disp,disp,i,xred,cell,eigvect.real,'sin',j)
  # e(R0+h-h)
  j = print_2d_deriv(disp,-disp,i,xred,cell,eigvect.real,'sin',j)
  # e(R0-h-h)
  j = print_2d_deriv(-disp,-disp,i,xred,cell,eigvect.real,'sin',j)

  # \xiI\xiI cos cos
  # e(R0+h+h)
  j = print_2d_deriv(disp,disp,i,xred,cell,eigvect.imag,'cos',j)
  # e(R0-h+h)
  j = print_2d_deriv(-disp,disp,i,xred,cell,eigvect.imag,'cos',j)
  # e(R0+h-h)
  j = print_2d_deriv(disp,-disp,i,xred,cell,eigvect.imag,'cos',j)
  # e(R0-h-h)
  j = print_2d_deriv(-disp,-disp,i,xred,cell,eigvect.imag,'cos',j)

  # \xiI\xiI sin sin
  # e(R0+h+h)
  j = print_2d_deriv(disp,disp,i,xred,cell,eigvect.imag,'sin',j)
  # e(R0-h+h)
  j = print_2d_deriv(-disp,disp,i,xred,cell,eigvect.imag,'sin',j)
  # e(R0+h-h)
  j = print_2d_deriv(disp,-disp,i,xred,cell,eigvect.imag,'sin',j)
  # e(R0-h-h)
  j = print_2d_deriv(-disp,-disp,i,xred,cell,eigvect.imag,'sin',j)

