# ----------------------------------------------------------------
# This python script is used to compute the contribution of the
# L point to the Gamma eigenenergies using finite difference.
# ---------------------------------------------------------------
import os
import numpy as np

def read_eigenenergies(datafile):
  flag1 = False
  flag2 = False
  with open(datafile,'r') as f:
    for line in f:
      if flag2:
        eigenenergies = np.append([eigenenergies],[map(float,line.split())])
	break
      if flag1:
        eigenenergies = map(float,line.split())
        flag2 = True
      if "kpt=  0.0000  0.0000  0.0000" in line:
        flag1 = True
  return eigenenergies

def use_cut3d(datafile,out,gamma,band):
  if datafile[len(datafile)-4:len(datafile)] == "VHXC":
    os.system("rm cut3d.files")
    with open("cut3d.files","a") as files:
      files.write(str(datafile)+"\n")
      files.write("1 \n")
      files.write("5 \n")  #3D formatted data
                           #(output the bare 3D data - one column)
      files.write(str(out)+"\n")
      files.write("0 \n")
    os.system("cut3d < cut3d.files ")
  if datafile[len(datafile)-3:len(datafile)] == "WFK":
    os.system("rm wfk.files")
    with open("wfk.files","a") as files:
      files.write(str(datafile)+"\n")
      files.write("1 \n")
      files.write("0 \n")
      files.write(str(gamma)+" \n") # Gamma point
      files.write(str(band)+" \n") # Band nb 6
      files.write("0 \n")
      files.write("0 \n")
      files.write("2 \n") # real 3D data one column
      files.write(str(out)+" \n")
      files.write("0 \n")
    os.system("cut3d < wfk.files ")

def integration(VHXC,wfk):
  psi_file=open(wfk,'r')
  v_file=open(VHXC,'r')
  S=0.0; N=0.0
  while True:
    psi_l=psi_file.readline()
    v_l=v_file.readline()
    if not psi_l or not v_l: break
    psi=float(psi_l)
    v=float(v_l)
    S+=psi*v*psi
    N+=psi*psi
  psi_file.close()
  v_file.close()
  S=S/N
  return S

# The base name for the supercell files are assumed to be named "supercell"
# and the primitive as assumed to be named "primitive"


# Value and number of the displacement
h = np.array([0.005])

# Initialisation values
k = 0

# Frequency (computed in DFPT using ABINIT)
omega = np.array([0.003915559,0.006088587,0.002163719,0.00589085])

# Carbon mass in a.u.
Mc = 21894.16693

# -------------------------------------------------------------
# Compute the second part the the NDDW term on a primitive cell
# -------------------------------------------------------------

# Level: Bandes
# HOMO
#lvl1 = 2 ; lvl2 = 3 ; lvl3 = 4
# LUMO
lvl1 = 5 ; lvl2 = 6 ; lvl3 = 7


# Prepare equilibrium files
use_cut3d("primitiveo_DS1_VHXC","EQ",0,0)
use_cut3d("primitiveo_DS1_WFK","wfk",1,lvl1)
use_cut3d("primitiveo_DS1_WFK","wfk",1,lvl2)
use_cut3d("primitiveo_DS1_WFK","wfk",1,lvl3)
averageEQ  = (integration("EQ","wfk_k1_b"+str(lvl1)+"_s1")+\
        integration("EQ","wfk_k1_b"+str(lvl2)+"_s1")+\
        integration("EQ","wfk_k1_b"+str(lvl3)+"_s1"))/3

k = 0
eigen = [0.0,0.0]
while k < 2:

# Transform binary files into .txt files using cut3d
  use_cut3d("primitiveo_DS"+str(12*k+2)+"_VHXC","VX++",0,0)
  use_cut3d("primitiveo_DS"+str(12*k+3)+"_VHXC","VX-+",0,0)
  use_cut3d("primitiveo_DS"+str(12*k+4)+"_VHXC","VX+-",0,0)
  use_cut3d("primitiveo_DS"+str(12*k+5)+"_VHXC","VX--",0,0)
  use_cut3d("primitiveo_DS"+str(12*k+6)+"_VHXC","VY++",0,0)
  use_cut3d("primitiveo_DS"+str(12*k+7)+"_VHXC","VY-+",0,0)
  use_cut3d("primitiveo_DS"+str(12*k+8)+"_VHXC","VY+-",0,0)
  use_cut3d("primitiveo_DS"+str(12*k+9)+"_VHXC","VY--",0,0)
  use_cut3d("primitiveo_DS"+str(12*k+10)+"_VHXC","VZ++",0,0)
  use_cut3d("primitiveo_DS"+str(12*k+11)+"_VHXC","VZ-+",0,0)
  use_cut3d("primitiveo_DS"+str(12*k+12)+"_VHXC","VZ+-",0,0)
  use_cut3d("primitiveo_DS"+str(12*k+13)+"_VHXC","VZ--",0,0)

# Perform the integration int(psi_0*VHXC*psi_0)
  averageVX1  = (integration("VX++","wfk_k1_b"+str(lvl1)+"_s1")+\
        integration("VX++","wfk_k1_b"+str(lvl2)+"_s1")+\
        integration("VX++","wfk_k1_b"+str(lvl3)+"_s1"))/3
  averageVX2  = (integration("VX-+","wfk_k1_b"+str(lvl1)+"_s1")+\
        integration("VX-+","wfk_k1_b"+str(lvl2)+"_s1")+\
        integration("VX-+","wfk_k1_b"+str(lvl3)+"_s1"))/3
  averageVX3  = (integration("VX+-","wfk_k1_b"+str(lvl1)+"_s1")+\
        integration("VX+-","wfk_k1_b"+str(lvl2)+"_s1")+\
        integration("VX+-","wfk_k1_b"+str(lvl3)+"_s1"))/3
  averageVX4  = (integration("VX--","wfk_k1_b"+str(lvl1)+"_s1")+\
        integration("VX--","wfk_k1_b"+str(lvl2)+"_s1")+\
        integration("VX--","wfk_k1_b"+str(lvl3)+"_s1"))/3
  averageVY1  = (integration("VY++","wfk_k1_b"+str(lvl1)+"_s1")+\
        integration("VY++","wfk_k1_b"+str(lvl2)+"_s1")+\
        integration("VY++","wfk_k1_b"+str(lvl3)+"_s1"))/3
  averageVY2  = (integration("VY-+","wfk_k1_b"+str(lvl1)+"_s1")+\
        integration("VY-+","wfk_k1_b"+str(lvl2)+"_s1")+\
        integration("VY-+","wfk_k1_b"+str(lvl3)+"_s1"))/3
  averageVY3  = (integration("VY+-","wfk_k1_b"+str(lvl1)+"_s1")+\
        integration("VY+-","wfk_k1_b"+str(lvl2)+"_s1")+\
        integration("VY+-","wfk_k1_b"+str(lvl3)+"_s1"))/3
  averageVY4  = (integration("VY--","wfk_k1_b"+str(lvl1)+"_s1")+\
        integration("VY--","wfk_k1_b"+str(lvl2)+"_s1")+\
        integration("VY--","wfk_k1_b"+str(lvl3)+"_s1"))/3
  averageVZ1  = (integration("VZ++","wfk_k1_b"+str(lvl1)+"_s1")+\
        integration("VZ++","wfk_k1_b"+str(lvl2)+"_s1")+\
        integration("VZ++","wfk_k1_b"+str(lvl3)+"_s1"))/3
  averageVZ2  = (integration("VZ-+","wfk_k1_b"+str(lvl1)+"_s1")+\
        integration("VZ-+","wfk_k1_b"+str(lvl2)+"_s1")+\
        integration("VZ-+","wfk_k1_b"+str(lvl3)+"_s1"))/3
  averageVZ3  = (integration("VZ+-","wfk_k1_b"+str(lvl1)+"_s1")+\
        integration("VZ+-","wfk_k1_b"+str(lvl2)+"_s1")+\
        integration("VZ+-","wfk_k1_b"+str(lvl3)+"_s1"))/3
  averageVZ4  = (integration("VZ--","wfk_k1_b"+str(lvl1)+"_s1")+\
        integration("VZ--","wfk_k1_b"+str(lvl2)+"_s1")+\
        integration("VZ--","wfk_k1_b"+str(lvl3)+"_s1"))/3

# Second order derivative (meV)
# LA mode
  eigen[k] = (averageVX1-averageVX2-averageVX3+averageVX4+\
          averageVY1-averageVY2-averageVY3+averageVY4+
          averageVZ1-averageVZ2-averageVZ3+averageVZ4)/\
           (4*(h[0]**2))*(1.0/(4*Mc*omega[1]))*27.211383*1000

  k += 1

print eigen

