# ----------------------------------------------------------------
# This python script is used to compute the matrix element
#  <n,k+q|H(1)|n',k>
# Here we specfically take n=1, n'=2 at gamma
# This correspond to k=gamma and q=L
# ---------------------------------------------------------------
import os
import numpy as np

def read_eigenenergies(datafile):
  flag = False
  with open(datafile,'r') as f:
    for line in f:
      if flag:
        eigenenergies = map(float,line.split())
        break
      if "kpt=  0.0000  0.0000  0.0000" in line:
        flag = True
  return eigenenergies

def use_cut3d(datafile,out,gamma,band):
  if datafile[len(datafile)-3:len(datafile)] == "POT":
    os.system("rm cut3d.files")
    with open("cut3d.files","a") as files:
      files.write(str(datafile)+"\n")
      files.write("1 \n")
      files.write("5 \n")  #3D formatted data
                           #(output the bare 3D data - one column)
      files.write(str(out)+"\n")
      files.write("0 \n")
    os.system("cut3d < cut3d.files ")
  if datafile[len(datafile)-3:len(datafile)] == "WFK":
    os.system("rm wfk.files")
    with open("wfk.files","a") as files:
      files.write(str(datafile)+"\n")
      files.write("1 \n")
      files.write("0 \n")
      files.write(str(gamma)+" \n") # Gamma point
      files.write(str(band)+" \n") # Band nb 6
      files.write("0 \n")
      files.write("0 \n")
      files.write("1 \n") # real 3D data one column
      files.write(str(out)+" \n")
      files.write("0 \n")
    os.system("cut3d < wfk.files ")

def first_order_wf(wfk0,wfk1,lam):
  S=0.0; N=0.0
  psi0 = np.loadtxt(wfk0).view(complex)
  psi1 = np.loadtxt(wfk1).view(complex)
  S = (psi1-psi0)/lam
#  np.savetxt('first_order_wfc.txt', S)
  return S

def integrationwf(wfk1,wfk2):
  S=0.0; N=0.0
  psi1 = np.loadtxt(wfk1).view(complex)
  if type(wfk2) != str:
    psi2 = wfk2
    S = psi1.conj()*psi2
    N1 = psi1.conj()*psi1
    return S.sum()/N1.sum()
  else:
    psi2 = np.loadtxt(wfk2).view(complex)
    S = psi1.conj()*psi2
    N1 = psi1.conj()*psi1
    return S.sum()/N1.sum()

h = np.array([0.01,0.005,0.0025,0.00125])
# Initialisation values
k = 0
# Frequency (computed in DFPT using ABINIT)
omega = np.array([0.00480084,0.005785943,0.002453274,0.005671174])
# Carbon mass in a.u.
Mc = 21894.16693

D = np.zeros([len(h),len(h),4])
# Prepare equilibrium files
# k=gamma
# q = L
use_cut3d("abo_DS1_WFK","wfkEQ",1,3) # psi_2^{(0)}
use_cut3d("abo_DS1_WFK","wfkEQ",1,1) # psi_2^{(0)}
energy2 = read_eigenenergies("abo_DS1_EIG")

while k < len(h):
  use_cut3d("abo_DS"+str(k+2)+"_WFK","wfk",1,1) # psi_1(lambda)

  psi1 = first_order_wf("wfkEQ_k1_b1_s1","wfk_k1_b1_s1",h[k])
  print "psi1", psi1
  LO = integrationwf("wfkEQ_k1_b3_s1",psi1)
  print "LO:", LO
  energy1 = read_eigenenergies("abo_DS"+str(k+2)+"_EIG")

# Define the phase
# phase = <psi1(0)|psi1(lambda)>/| <psi1(0)|psi1(lambda)>|
  phase_int = integrationwf("wfkEQ_k1_b1_s1","wfk_k1_b1_s1")
  phase_mod = np.abs(phase_int)
  phase = phase_int/phase_mod

  elph=(energy1[0]-energy2[2])*LO/phase
  print "elph",elph
  D[0,k,0] = elph
  k += 1

l = 0
while l < len(h):
  k = 0
  while k < len(h):
    if (l-1) >= 0 and (k-1) >= 0:
      D[l,k,0] = (D[l-1,k,0]-(D[l-1,k-1,0]*(1.0/(2**(2*l)))))/\
                 (1-(1.0/(2**(2*l))))
      if D[l-1,k-1,0] == 0.0:
        D[l,k,0] = 0.0
    k += 1
  l += 1

print "Richardson extrapolation of <psi_2^{(0)}|dVscf|psi_1^{(0)} for LO"
print D
